
#include <stdio.h>
#include <conio.h>
#include "SimpleBits.h"

#define FOURCC( ch0, ch1, ch2, ch3 )				\
		( (unsigned long)(unsigned char)(ch0) | ( (unsigned long)(unsigned char)(ch1) << 8 ) |	\
		( (unsigned long)(unsigned char)(ch2) << 16 ) | ( (unsigned long)(unsigned char)(ch3) << 24 ) )

typedef 
enum AAC_TYPE
{
	AAC_TYPE_ADIF,
	AAC_TYPE_ADTS,
	AAC_TYPE_UNKNOWN
};

void main(int argc, char *argv[])
{
	if ( argc < 3 || argc > 4 )
	{
		printf("Nic's Patcher for AAC Files v0.1\n\n");
		printf("Usage: AACPatch input.aac output.aac [-2]\n");
		printf("       (-2 option will convert to MPEG-2)\n\nWill NOT remove emphasis field!\nDont use with aacenc_mpeg4.exe!\n");
		return;
	}

	const char *szFile = argv[1];
	const char *szOutFile = argv[2];

	bool bGotoMPEG2 = false;

	if ( argc == 4 )
		bGotoMPEG2 = true;

	// check for type of AAC file
	CSimpleBits		Bits;
	if ( !Bits.SetFileName(szFile) )
	{
		printf("Input File Not Found!\n");
		return;
	}

	FILE *fout = NULL;
	unsigned long bits;
	Bits.ReadBytes(&bits, 4);
	
	unsigned char *OutBuf = new unsigned char[10000];

	int nAACType = AAC_TYPE_UNKNOWN;
	
	if ( bits == FOURCC('A', 'D', 'I', 'F') )
	{
		nAACType = AAC_TYPE_ADIF;
		printf("Type appears to be ADIF and not supported\n");
	}
	else
	{
		bool bFirst = true;

		int sync;
		int id;
		int layer;
		int protection_absent;
		int profile;
		int sfreq_index;
		int private_bit;
		int channel_configuration;
		int original_copy;
		int home;
		int copyright_identification_bit;
		int copyright_identification_start;
		int aac_frame_length;
		int adts_buffer_fullness;
		int no_raw_data_blocks_in_frame;
		int crc_check;

		fout = fopen(szOutFile, "wb");

		Bits.Seek(0);
		long nCurrentPos = 0;

		while ( true )
		{
			sync = Bits.ReadBits(12);

			if ( Bits.EndOfFile() )
			{
				printf("End Of File! All Done!\n");
				break;
			}

			if ( sync == 0xfff )
			{
				nAACType = AAC_TYPE_ADTS;
				
				id								= Bits.ReadBits(1);		
				layer							= Bits.ReadBits(2);		
				protection_absent				= Bits.ReadBits(1);		
				profile							= Bits.ReadBits(2);		
				sfreq_index						= Bits.ReadBits(4);		
				private_bit						= Bits.ReadBits(1);							
				channel_configuration			= Bits.ReadBits(3);							
				original_copy					= Bits.ReadBits(1);
				home							= Bits.ReadBits(1);


				copyright_identification_bit	= Bits.ReadBits(1);
				copyright_identification_start	= Bits.ReadBits(1);
				aac_frame_length				= Bits.ReadBits(13);
				adts_buffer_fullness			= Bits.ReadBits(11);
				no_raw_data_blocks_in_frame		= Bits.ReadBits(2);

				if ( protection_absent == 0 )
					crc_check = Bits.ReadBits(16);

				if ( bFirst )
				{
					printf("Input is AAC MPEG-%d ", id == 0 ? 4 : 2);
					switch ( profile )
					{
					case 0:
						printf("Main Profile");
						break;
					case 1: 
						printf("Low Complexity (LC)");
						break;
					case 2:
						printf("Scalable Sample Rate (SSR)");
						break;
					case 3:
						printf("Long Term Prediction (LTP MPEG-4 Only)");
						break;
					}
					printf("\n");
					printf("Creating AAC MPEG-%d ", bGotoMPEG2 == false ? 4 : 2);
					switch ( profile )
					{
					case 0:
						printf("Main Profile");
						break;
					case 1: 
						printf("Low Complexity (LC)");
						break;
					case 2:
						printf("Scalable Sample Rate (SSR)");
						break;
					case 3:
						printf("Long Term Prediction (LTP MPEG-4 Only)");
						break;
					}
					printf("\n");
					bFirst = false;
				}
						
				// Write it out. go back to start read the frame in, correct the bits, write out the file
				Bits.Seek(nCurrentPos);
				Bits.ReadBytes(OutBuf, aac_frame_length);

				// Make 13th bit MPEG-4 (0) and then spit it out
				if ( !bGotoMPEG2 )
					OutBuf[1] &= 0xf7;
				else
					OutBuf[1] |= 8;

				fwrite(OutBuf, 1, aac_frame_length, fout);

				nCurrentPos += aac_frame_length;
				Bits.Seek(nCurrentPos);
			}
			else
			{
				printf("Failed (sync bits not found)...exiting!\n");
				break;
			}
		}
	}

	if ( fout )
		fclose(fout);
	delete [] OutBuf;
}
	


